package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.EvaluationRevisionMapper;
import com.ims.eval.dao.ResponsibilityIndicatorInfoMapper;
import com.ims.eval.entity.EvaluationRevision;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.ResponsibilityIndicatorInfo;
import com.ims.eval.entity.dto.response.IndicatorResDTO;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.service.IEvaluationRevisionService;
import com.ims.eval.service.IIndicatorService;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author hlf
 * @date 2023/5/23 10:34
 * 文件说明：
 */
@Service
public class EvaluationRevisionServiceImpl extends ServiceImpl<EvaluationRevisionMapper, EvaluationRevision> implements IEvaluationRevisionService {

	@Autowired
	private IIndicatorService indicatorService;

	@Resource
	private ResponsibilityIndicatorInfoMapper responsibilityIndicatorInfoMapper;

	@Autowired
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;

	@Override
	public IPage<EvaluationRevision> listPage(Integer pageNum, Integer pageSize, String id, String responsibilityCode, String cycleUnit, List<String> checkCycle, String beginDate, String endDate, String stage, String createBy, String year, String month, String des) {

		QueryWrapper<EvaluationRevision> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluationRevision> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(EvaluationRevision::getId, id);
		}

		if (StringUtils.isNotEmpty(responsibilityCode)) {
			qw.lambda().like(EvaluationRevision::getResponsibilityCode, responsibilityCode);
		}

		if (StringUtils.isNotEmpty(cycleUnit)) {
			qw.lambda().eq(EvaluationRevision::getCycleUnit, cycleUnit);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().in(EvaluationRevision::getCheckCycle, checkCycle);
		}

		if (StringUtils.isNotEmpty(beginDate) && StringUtils.isNotEmpty(endDate)) {
			qw.lambda().ge(EvaluationRevision::getBeginDate, beginDate);
			qw.lambda().le(EvaluationRevision::getEndDate, endDate);
		}

		if (StringUtils.isNotEmpty(stage)) {
			qw.lambda().eq(EvaluationRevision::getStage, stage);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(EvaluationRevision::getCreateBy, createBy);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(EvaluationRevision::getCreateBy, createBy);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(EvaluationRevision::getYear, year);
		}

		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().eq(EvaluationRevision::getMonth, month);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(EvaluationRevision::getDes, des);
		}

		qw.lambda().orderByDesc(EvaluationRevision::getCreateTime);

		return baseMapper.selectPage(page, qw);
	}

	@Override
	public boolean saveOrUpdate(EvaluationRevision evaluationRevision) {
		String id;
		if (StringUtils.isEmpty(evaluationRevision.getId())) {//true为空
			id = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
			QueryWrapper<EvaluationRevision> qw = new QueryWrapper<>();
			qw.lambda().eq(EvaluationRevision::getOrganizationType, "DWKP");
			qw.lambda().eq(EvaluationRevision::getCheckCycle, "NDKP");
			if (StringUtils.isNotEmpty(evaluationRevision.getYear())) {
				qw.lambda().eq(EvaluationRevision::getYear, evaluationRevision.getYear());
			}
			int count = super.count(qw);
			String responsibilityCode = "MBXD_GDDL_" + evaluationRevision.getYear() + StringUtils.addZeroForStr(String.valueOf(count + 1), 2, "l");
			evaluationRevision.setId(id);
			evaluationRevision.setResponsibilityCode(responsibilityCode);
			evaluationRevision.setStage("流程未启动");
			evaluationRevision.setCreateTime(new Date());
		} else {
			id = evaluationRevision.getId();
		}
		if (StringUtils.isNotEmpty(evaluationRevision.getOrganizationId()) && StringUtils.isNotEmpty(evaluationRevision.getOrganizationName())) {
			String[] organizationIds = evaluationRevision.getOrganizationId().split(",");
			for (String organizationId : organizationIds) {
				List<ResponsibilityIndicatorInfo> responsibilityIndicatorInfoList = responsibilityIndicatorInfoService.selectReviseResponsibilityIndicatorInfoList(organizationId, String.valueOf(Calendar.getInstance().get(Calendar.YEAR)));
				responsibilityIndicatorInfoList.forEach(p -> p.setReviseId(id));
				responsibilityIndicatorInfoService.updateBatchById(responsibilityIndicatorInfoList);
			}
		}
		return super.saveOrUpdate(evaluationRevision);
	}

	@Override
	public boolean removeByIds(List<String> ids) {
		for (String id : ids) {
			EvaluationRevision evaluationRevision = super.getById(id);
			if ("流程未启动".equals(evaluationRevision.getStage())) {
				baseMapper.deleteById(id);
			}
		}
		return true;
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		EvaluationRevision evaluationRevision = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			if ("考评委员会审核".equals(state)) {
				state = "流程已结束";
				QueryWrapper<ResponsibilityIndicatorInfo> qw = new QueryWrapper<>();
				if (StringUtils.isNotEmpty(evaluationRevision.getId())) {
					qw.lambda().eq(ResponsibilityIndicatorInfo::getReviseId, evaluationRevision.getId());
				}
				List<ResponsibilityIndicatorInfo> list = new ArrayList<>();
				List<ResponsibilityIndicatorInfo> responsibilityIndicatorInfoList = responsibilityIndicatorInfoService.list(qw);
				for (ResponsibilityIndicatorInfo responsibilityIndicatorInfo : responsibilityIndicatorInfoList) {
					if ("1-1".equals(responsibilityIndicatorInfo.getDataState())) {
						responsibilityIndicatorInfo.setDataState("1");
					} else if ("0-0".equals(responsibilityIndicatorInfo.getDataState())) {
						responsibilityIndicatorInfo.setDataState("0");
					}
					list.add(responsibilityIndicatorInfo);
				}
				responsibilityIndicatorInfoService.updateBatchById(list);
			}
			evaluationRevision.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			evaluationRevision.setInstId(instId);
		}
		return super.saveOrUpdate(evaluationRevision);
	}

	@Override
	public Map getResponsibilityIndicatorInfoList(String reviseId, String indicatorId, String organizationShortName, String binSection, String binStage, HttpServletRequest request) {
		Map data = new HashMap();
		//设置数据头
		//获取指标名称
		boolean titlemark = true;//标记生成标题
		List<Map> indicatorMap = getResponsibilityIndicatorList(reviseId, binSection, binStage, request);
		List<String> idList = indicatorMap.stream()
			.map(map -> String.valueOf(map.get("id"))) // 提取每个Map中key为"id"的值，并转换成String
			.collect(Collectors.toList()); // 将提取的值收集到List中
		if (null == idList || idList.size() <= 0) {
			//throw new CustomException("未查询到数据");
			data.put("title", null);
			return data;
		}
		List<IndicatorResDTO> indicatorResDTOS = indicatorService.getGroupChildCodeByIds(idList);
		if (titlemark) {
			// 遍历分组后的Map
			Map title = new LinkedHashMap();//保存所有的指标名
			for (IndicatorResDTO dto : indicatorResDTOS) {
				List<Map> titleArray = new ArrayList<>();
				// 处理每个分组中的元素
				Map titlemap = new LinkedHashMap();
				titlemap.put("childName", dto.getChildName());//子指标名
				titlemap.put("key", dto.getDeptName());//名称
				titlemap.put("code", dto.getChildCode() + "_" + dto.getOptionCode());//名称
				titlemap.put("flag", dto.getIsQuantified());//是否量化
				titleArray.add(titlemap);
				title.put(dto.getChildName(), titleArray);
			}
			//状态标题头
			Map deptstateMap = new LinkedHashMap();
			List<Map> titlestateArray = new ArrayList<>();
			deptstateMap.put("key", "填报状态");//名称
			deptstateMap.put("code", "state");//名称
			deptstateMap.put("flag", false);//是否编辑
			titlestateArray.add(deptstateMap);
			title.put("状态", titlestateArray);

			data.put("title", title);
			titlemark = false;
		}
		//获取数据
		List<Map> datamapList = new ArrayList<>();
		if (null != organizationShortName) {
			organizationShortName = organizationShortName.trim();
		}
		if (null != organizationShortName) {
			organizationShortName = organizationShortName.trim();
		}
		List<ResponsibilityIndicatorInfoResDTO> dataList = responsibilityIndicatorInfoMapper.getResponsibilityInfoList("", indicatorId, organizationShortName, binSection, binStage, "1-1", reviseId);
		Map<String, List<ResponsibilityIndicatorInfoResDTO>> groupeddataMap = dataList.stream().collect(Collectors.groupingBy(ResponsibilityIndicatorInfoResDTO::getOrganizationId)); // 根据organizationId进行分组
		// 遍历分组后的Map
		for (Map.Entry<String, List<ResponsibilityIndicatorInfoResDTO>> entry : groupeddataMap.entrySet()) {
			List<ResponsibilityIndicatorInfoResDTO> groupedList = entry.getValue(); // 获取分组后的List
			Map<String, ResponsibilityIndicatorInfoResDTO> resultMap = groupedList.stream()
				.collect(Collectors.toMap(
					dto -> dto.getChildCode() + "_" + dto.getOptionCode(),
					dto -> dto,
					(oldValue, newValue) -> oldValue)); // 解决键重复的情况
			Map indicatormap = new HashMap();
			// 进一步处理每个分组
			for (IndicatorResDTO d : indicatorResDTOS) {
				boolean mark = true;//标记给公司名赋值
				if (mark) {
					indicatormap.put("organizationShortName", resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getOrganizationShortName());//公司名
					indicatormap.put("organizationId", resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getOrganizationId());//公司id
					indicatormap.put("state", resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getState());
					indicatormap.put("IS_LH_state", "3");
					mark = false;
				}
				//指标属性
				indicatormap.put(d.getChildCode() + "_" + d.getOptionCode(), null == resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getNonQuantifiedValue() ? "" : resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getNonQuantifiedValue());
				//指标id
				indicatormap.put("ID_" + d.getChildCode() + "_" + d.getOptionCode(), resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getId());
				//是否量化
				indicatormap.put("IS_LH_" + d.getChildCode() + "_" + d.getOptionCode(), resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getIsQuantified() ? "1" : "2");
			}
			datamapList.add(indicatormap);
		}
		data.put("value", datamapList);
		return data;
	}

	private List<Map> getResponsibilityIndicatorList(String reviseId, String binSection, String binStage, HttpServletRequest request) {
		//判断是否是（重点专项和管理事项）
		List<Map> indicatorMap = new ArrayList<>();
		if ("ZDZX".equals(CacheContext.bseIdObject.get(binStage).getStageCode()) || "KJCX".equals(CacheContext.bseIdObject.get(binStage).getStageCode())) {
			EvaluationRevision dby = baseMapper.selectById(reviseId);
			if (null == dby) {
				return null;
			}
			List<Indicator> indicatorList = indicatorService.listAll("", "", "", binSection, binStage, "", "", dby.getCheckCycle());
			if (null != indicatorList && indicatorList.size() > 0) {
				for (Indicator i : indicatorList) {
					Map map = new HashMap();
					map.put("id", i.getId());
					map.put("indicator_name", i.getIndicatorName());
					indicatorMap.add(map);
				}
			}
		} else {
			indicatorMap = responsibilityIndicatorInfoMapper.selectResponsibilityIndicatorList("", binSection, binStage, reviseId);
		}
		return indicatorMap;
	}
}
